#include "arrayfuncs.h"
#include "vecfunc.h"
#include "c3dtypes.h"
#include "raylib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int TestFunc(int x) {
    x = x + 1;
    return x;
}

Vector3 BreakDownObjTriElement(char *vert) {
    int vertex = 0;
    int vertextexture = 0;
    int normal = 0;

    sscanf(vert, "%i/%i/%i", &vertex, &vertextexture, &normal);
    return (Vector3){vertex, vertextexture, normal};
}

CMaterial NewCmat(char *matname, char *fname) {
    CMaterial cmat;
	cmat.name = malloc(20 * sizeof(char));
	cmat.texturefile = malloc(20 * sizeof(char));
    strncpy(cmat.name, matname, 15);
    strncpy(cmat.texturefile, fname, 15);
    Image teximg = LoadImage(fname);
	ImageFlipVertical(&teximg);
	cmat.h = teximg.height;
	cmat.w = teximg.width;
    cmat.tex = LoadImageColors(teximg);
    printf("Loaded SingleTex %s with h:%d and w:%d\n", matname,cmat.h,cmat.w);
    return cmat;
}

void LoadMats(char *fname, CMaterialArray *cmats) {
    FILE *f = fopen(fname, "r");

    char mtlname[50];
    strncpy(mtlname, "", 50);
    char mtltex[50];
    strncpy(mtltex, "", 50);
	printf("Loading mat of name: %s\n", fname);
    while (true) {
        char t[500];
        char *fgetres = fgets(t, 500, f);

        char objtype[30];
        char v1[100];
        char v2[100];
        char v3[100];
        sscanf(t, "%s %s %s %s", objtype, v1, v2, v3);

        printf("Entering comp for mtlname: %s, mtltex: %s\n", mtlname, mtltex);

        if (fgetres == NULL) {
            if ((strcmp(mtlname, "") != 0) && (strcmp(mtltex, "") != 0)) {
                cmats->arr[cmats->length] = NewCmat(mtlname, mtltex);
				printf("Younki Sploinky\n");
                cmats->length = cmats->length + 1;
                strncpy(mtltex, "", 50);
                printf("ended and added\n");
            }
            break;
        }

        if (strcmp(objtype, "newmtl") == 0) {
            if ((strcmp(mtlname, "") != 0) && (strcmp(mtltex, "") != 0)) {
                cmats->arr[cmats->length] = NewCmat(mtlname, mtltex);
                cmats->length = cmats->length + 1;
                strncpy(mtltex, "", 50);
            }
            strncpy(mtlname, v1, 50);
        }
        if (strcmp(objtype, "map_Kd") == 0) {
            strncpy(mtltex, v1, 50);
        }
    }
    printf("Loaded MatFile %s\n", fname);
    fclose(f);
}

CMaterial * FindCmat(CMaterialArray * cmats, char * name){
	for (int i = 0; i < cmats->length; i++) {
		if (strcmp(cmats->arr[i].name, name) == 0){
			return &cmats->arr[i];
		}
	}
	return NULL;
}

Object3D ReadObjectFromFile(char *fname) {
    Object3D out;
    Vector3 *VertexArray = malloc(1000000 * sizeof(Vector3));
    int VertexArrayLength = 0;

    Vector2 *UVArray = malloc(1000000 * sizeof(Vector2));
    int UVArrayLength = 0;

    TriArray * tarr = malloc(sizeof(TriArray));
    tarr->arr = malloc(1000000 * sizeof(Tri));
    tarr->length = 0;

    CMaterialArray * cmats = malloc(sizeof(CMaterialArray));
    cmats->arr = malloc(500 * sizeof(CMaterial));
	cmats->length = 0;

    out.triangles = tarr;
	out.mats = cmats;

    strncpy(out.name, fname, 100);

    FILE *f = fopen(fname, "r");

	
		CMaterial * currentmat = NULL;
	
    while (true) {
        char t[500];
        char *fgetres = fgets(t, 500, f);
        if (fgetres == NULL) {
            break;
        }

		
        char objtype[10];
        char v1[100];
        char v2[100];
        char v3[100];
        sscanf(t, "%s %s %s %s", objtype, v1, v2, v3);

        if (strcmp(objtype, "mtllib") == 0) {
            LoadMats(v1, cmats);
        }

        if (strcmp(objtype, "v") == 0) {
            VertexArray[VertexArrayLength].x = atof(v1) * 100;
            VertexArray[VertexArrayLength].y = atof(v2) * 100;
            VertexArray[VertexArrayLength].z = atof(v3) * 100;
            VertexArrayLength = VertexArrayLength + 1;
        }
        if (strcmp(objtype, "vt") == 0) {
            //printf("vt: (%s,%s)\n", v1, v2);
            UVArray[UVArrayLength].x = atof(v1);
            UVArray[UVArrayLength].y = atof(v2);
			//Vector2Print(UVArray[UVArrayLength]);
            UVArrayLength = UVArrayLength + 1;
        }
		if (strcmp(objtype, "usemtl") == 0) {
			currentmat = FindCmat(cmats, v1);
			printf("CHANGED MAT TO: %s %s\n", currentmat->name, currentmat->texturefile);
		}
        if (strcmp(objtype, "f") == 0) {
			int slashCount = 0;
			for (int i = 0; i < strlen(t); i++) {
				if (t[i] == '/') {
					slashCount = slashCount + 1;
				}
			}

			char blank[100];
			char v1t[100];
			char v2t[100];
			char v3t[100];

			if (slashCount > 2) {
				sscanf(v1, "%[^/]%s", blank, v1t);
				sscanf(v2, "%[^/]%s", blank, v2t);
				sscanf(v3, "%[^/]%s", blank, v3t);
			}
			if (slashCount > 5) {
				sscanf(v1, "%[^/]/%[^/]%s", blank, v1t, blank);
				sscanf(v2, "%[^/]/%[^/]%s", blank, v2t, blank);
				sscanf(v3, "%[^/]/%[^/]%s", blank, v3t, blank);
			}
            // TODO: append face to triarry
            Tri temptri;

            temptri.a = VertexArray[atoi(v1) - 1];
            temptri.b = VertexArray[atoi(v2) - 1];
            temptri.c = VertexArray[atoi(v3) - 1];

			temptri.auv = UVArray[atoi(v1t)-1];
			temptri.buv = UVArray[atoi(v2t)-1];
			temptri.cuv = UVArray[atoi(v3t)-1];
			//printf("%s\n",v1t);

			//Vector2Print(temptri.cuv);
			
			temptri.material = currentmat;

            // printf("OLINE: %s\n", t);
            // printf("adding tri A: %d, B: %d, C: %d\n", atoi(v1), atoi(v2) , atoi(v3) );

            temptri.color = GREEN;
            TriArrayAppend(tarr, temptri);
        }
        // need to bring in triarray functions into their own file
    }

    fclose(f);
    printf("Loaded model %s with %d tris\n", fname, tarr->length);

	printf("Mats Loaded\n");
	for (int i = 0; i < cmats->length; i++) {
		printf("%s\n", cmats->arr[i].name);
	}
    return out;

    // TODO: use file object and read each line
    //  add verticies to the vertex list, +1 ing the length value each time
    //  just fucking parse the obj file its not that hard
}
